#![cfg(target_os = "android")]

/*use jni::{
    objects::{JClass, JString},
    sys::{jboolean, jchar, jint, jstring},
    JNIEnv,
};
use std::{error::Error, fs, net::Ipv4Addr, str};
use clap::{App, Arg, ArgMatches};
use env_logger::Builder;
use log::{error, LevelFilter};
use crate::config::{ ServerConfiguration, ClientConfiguration, ObfsProtocol, ServerPeer };
use fast32::base32::RFC4648;
use crossbeam_channel::unbounded;*/

use ::jni::objects::GlobalRef;
use ::jni::JavaVM;
use robusta_jni::bridge;
use std::sync::OnceLock;

mod config;
mod client;
mod udp;
mod mobile;
mod simple_log;

/*
#[no_mangle]
pub unsafe extern "C" fn Java_com_alterdekim_frida_FridaLib_run(
    mut env: JNIEnv,
    _clazz: JClass,
    config_b32: JString,
    tun_fd: jint,
    close_fd_on_drop: jboolean,
) -> jint {
    let config = env.get_string(config_b32).unwrap().into();
    let close_fd_on_drop = close_fd_on_drop != 0;
    mobile::mobile_run(config, close_fd_on_drop, tun_fd)
}

#[no_mangle]
pub unsafe extern "C" fn Java_com_alterdekim_frida_FridaLib_fetchLogs(
    mut env: JNIEnv,
    _clazz: JClass
) -> jstring {
    **env.new_string(String::from_utf8_lossy(&simple_log::fetch_logs()[..]).to_string()).unwrap()
}

#[no_mangle]
pub unsafe extern "C" fn Java_com_alterdekim_frida_FridaLib_stop(_env: JNIEnv, _: JClass) -> jint {
    mobile::mobile_stop()
}*/

#[bridge]
mod jni {
    use android_logger::Config;
    use jni::objects::{GlobalRef, JObject, JValue, JString};
    use jni::sys::{jboolean, jchar, jint, jstring};
    use log::{info, trace};
    use robusta_jni::convert::{IntoJavaValue, Signature, TryFromJavaValue, TryIntoJavaValue};
    use robusta_jni::jni::errors::Result as JniResult;
    use robusta_jni::jni::objects::AutoLocal;
    use robusta_jni::jni::JNIEnv;

    #[derive(Signature, TryIntoJavaValue, IntoJavaValue, TryFromJavaValue)]
    #[package(com.alterdekim.frida)]
    pub struct FridaLib<'env: 'borrow, 'borrow> {
        #[instance]
        raw: AutoLocal<'env, 'borrow>,
    }

    impl<'env: 'borrow, 'borrow> FridaLib<'env, 'borrow> {
        pub extern "jni" fn start(self, env: &JNIEnv, config_b32: String,
            tun_fd: i32,
            close_fd_on_drop: bool) -> JniResult<i32> {
            android_logger::init_once(Config::default().with_tag("RustFrida"));
            
            FridaLib::traceFromNative(env, "Hello world");

            Ok(-2)
        }

        pub extern "jni" fn stop(self, env: &JNIEnv) -> JniResult<i32> {
            Ok(-1)
        }

        pub extern "java" fn traceFromNative(
            env: &JNIEnv,
            text: String,
        ) {}

        /*pub extern "java" fn threadTestNoClass(env: &JNIEnv, s: String) -> JniResult<i32> {}
        pub extern "java" fn threadTestWithClass(
            env: &JNIEnv,
            class_ref: &GlobalRef,
            s: String,
        ) -> JniResult<i32> {
        }*/
    }
}


 /*
#[tokio::main]
async fn main() {
    // Initialize the logger with 'info' as the default level
    Builder::new()
        .filter(None, LevelFilter::Info)
        .init();

    let matches = App::new("Frida")
        .version("0.1.2")
        .author("alterwain")
        .about("VPN software (android port)")
        .arg(Arg::with_name("config")
            .long("config")
            .required(true)
            .value_name("B32_RAW")
            .help("Configuration file data (base32 encoded)")
            .takes_value(true))
        .arg(Arg::with_name("fd")
            .long("fd")
            .required(true)
            .value_name("INT")
            .help("File descriptor int")
            .takes_value(true))
        .get_matches();

    let cfg_raw = matches.value_of("config").unwrap();

    let config: ClientConfiguration = serde_yaml::from_slice(RFC4648.decode(cfg_raw.as_bytes()).unwrap().as_slice()).expect("Bad client config file structure");

    client::client_mode(config, matches.value_of("fd").unwrap().parse().unwrap()).await;
}
    */